/**
 * \file main.cc
 * \author Scott F. H. Kaplan <sfkaplan@cs.amherst.edu>
 * \date December 2002
 * \brief The code for raw-reference-binary-to-text.
 *
 * This file contains all of the code for
 * raw-reference-binary-to-text, which converts the binary output of
 * the Laplace raw reference trace into a simple text format.  Note
 * that this version is not optimized:  It relies on printf() to
 * produce the output, and performs no buffering of its own output.
 * Such optimizations should be applied later, as reference handling
 * is the key bottleneck to trace collection.
 */



// ===================================================================
#include <stdio.h>
#include <stdlib.h>
// ===================================================================



// =================================================================
// TYPES
// Definitions of types for each field in the raw reference trace.
// This is taken from tracer.cc in Laplace itself.

/** \brief The type for the tag that indicates the type of a
 *         reference. */
typedef char tag_t;

/** \brief The type for the cycle counter timestamp. */
typedef unsigned long long int timestamp_t;

/** \brief The type for length values. */
typedef unsigned char length_t;

/** \brief The type for the address space identifier, a.k.a. the page
 *         number holding the base level of the page table. */
typedef unsigned long int address_space_ID_t;

/** \brief The type for virtual addresses. */
typedef unsigned long int address_t;

/** \brief A record structure in which a reference record will be
 *         passed and stored. */
struct reference_record_s {

  tag_t tag;
  timestamp_t timestamp;
  length_t length;
  address_space_ID_t address_space_ID;
  address_t virtual_address;

};
// =================================================================



// =================================================================
// GLOBALS and CONSTANTS

const tag_t TAG_END_OF_TRACE = 'e';
// ===================================================================



// ===================================================================
/**
 * \brief The entry point.
 * 
 * \param argc The number of arguments given at the command line.
 * \param argv The vector of argument strings passed at the command
 *             line.
 * \return An execution result code.
 * 
 * Parse the command line arguments.  For this utility, there should
 * be none, as input is taken from stdin, and output is sent to
 * stdout.
 */

int
main (const unsigned int argc, const char** const argv) {

  // Were the correct number of arguments passed?
  if (argc != 1) {

    // No.  Emit usage and exit.
    fprintf(stderr, "Usage: %s <NO ARGUMENTS>\n", argv[0]);
    return 1;

  }

  // Allocate a single reference record into which the binary source
  // will be read, one record at a time.
  reference_record_s binary_record;

  // Read binary records until the end of the trace has been reached.
  do {

    // Attempt to read the next record.
    size_t read_result = fread(&binary_record,
			       sizeof(binary_record),
			       1,
			       stdin);

    // Ensure that the read succeeded; otherwise, bail out.
    if (read_result != 1) {

      if (feof(stdin)) {
	fprintf(stderr, "Premature EOF\n");
      } else {
	fprintf(stderr, "Failed in reading binary record\n");
      }

      exit(1);

    }

    // Emit the binary record as text.  If the record is the
    // end-of-trace record, then emit just the tag.
    if (binary_record.tag == TAG_END_OF_TRACE) {

      printf("%c\n", binary_record.tag);

    } else {

      printf("%c %qx %hx %lx %lx\n",
	     binary_record.tag,
	     binary_record.timestamp,
	     binary_record.length,
	     binary_record.address_space_ID,
	     binary_record.virtual_address);

    }

  } while (binary_record.tag != TAG_END_OF_TRACE);

  // Exit successfully.
  return 0;

} // main
// ===================================================================
